@@ -10,7 +10,10 @@ utils::globalVariables(c(
   "Sum", "Target", "Test_End", "Train_End", "Train_Test_ID", "Type", "Variable", "as2",
   "combo_list", "data", "get_export_packages", "hi_80", "hi_95", "i", "lo_80", "lo_95",
   "model_spec_1", "name", "path_ext", "predict", "read.csv", "sc", "weighted_MAPE", "where",
-  "x", "num_cores", "run_info", "negative_forecast", "Forecast_Adj", "Final_Col", "lag_val", "libs"
+  "x", "num_cores", "run_info", "negative_forecast", "Forecast_Adj", "Final_Col", "lag_val", "libs",
+  ".config", "Forecast_Tbl", "Model_Workflow", "id", "model_run",
+  "Auto_Accept", "Feature", "Imp", "Importance", "LOFO_Var", "Var_RMSE", "Vote", "Votes", "desc",
+  "term", "Column", "Box_Cox_Lambda", "get_recipie_configurable"
 ))
 
 #' @importFrom magrittr %>%
